@@ -526,6 +526,7 @@ static void gui_init(void)
         gui = (gui_t*)calloc(1, sizeof(*gui));
         init_ImGui();
         gui->gestures.drag.type = GESTURE_DRAG;
+        gui->gestures.drag.button = GESTURE_LMB;
         gui->gestures.drag.callback = on_gesture;
         gui->gestures.hover.type = GESTURE_HOVER;
         gui->gestures.hover.callback = on_gesture;
@@ -584,13 +585,15 @@ static int check_action_shortcut(action_t *action, void *user)
         if (!str_startswith(s, "Ctrl")) return 0;
         s += strlen("Ctrl ");
         check_char = false;
+    } else {
+        if (str_startswith(s, "Ctrl")) return 0;
     }
     if (io.KeyShift) {
         check_key = false;
     }
     if (    (check_char && isCharPressed(s[0])) ||
             (check_key && ImGui::IsKeyPressed(s[0], false))) {
-        action_exec(action, "");
+        action_exec(action);
         return 1;
     }
     return 0;
@@ -667,7 +670,7 @@ void gui_iter(const inputs_t *inputs)
 
     io.DisplayFramebufferScale = ImVec2(goxel.screen_scale,
                                         goxel.screen_scale);
-    io.DeltaTime = 1.0 / 60;
+    io.DeltaTime = goxel.delta_time;
     gui->inputs = inputs;
 
     if (inputs) {
@@ -746,7 +749,7 @@ void gui_iter(const inputs_t *inputs)
 
     // Handle the shortcuts.  XXX: this should be done with actions.
     if (ImGui::IsKeyPressed(KEY_DELETE, false))
-        action_exec2("layer_clear", "");
+        action_exec2(ACTION_layer_clear);
 
     if (!io.WantCaptureKeyboard) {
         float last_tool_radius = goxel.tool_radius;
@@ -1019,12 +1022,10 @@ bool gui_angle(const char *id, float *v, int vmin, int vmax)
     return ret;
 }
 
-bool gui_action_button(const char *id, const char *label, float size,
-                       const char *sig, ...)
+bool gui_action_button(int id, const char *label, float size)
 {
     bool ret;
     const action_t *action;
-    va_list ap;
 
     action = action_get(id, true);
     assert(action);
@@ -1032,32 +1033,12 @@ bool gui_action_button(const char *id, const char *label, float size,
     ret = gui_button(label, size, action->icon);
     if (IsItemHovered()) goxel_set_help_text(action_get(id, true)->help);
     if (ret) {
-        va_start(ap, sig);
-        action_execv(action_get(id, true), sig, ap);
-        va_end(ap);
+        action_exec(action_get(id, true));
     }
     PopID();
     return ret;
 }
 
-bool gui_action_checkbox(const char *id, const char *label)
-{
-    bool b;
-    const action_t *action = action_get(id, true);
-    action_exec(action, ">b", &b);
-    if (ImGui::Checkbox(label, &b)) {
-        action_exec(action, "b", b);
-        return true;
-    }
-    if (ImGui::IsItemHovered()) {
-        if (!*action->shortcut)
-            goxel_set_help_text(action->help);
-        else
-            goxel_set_help_text("%s (%s)", action->help, action->shortcut);
-    }
-    return false;
-}
-
 static bool _selectable(const char *label, bool *v, const char *tooltip,
                         float w, int icon, int group)
 {
@@ -1352,7 +1333,7 @@ bool gui_combo(const char *label, int *v, const char **names, int nb)
     return ret;
 }
 
-bool gui_combo_begin(const char *label, const void *current)
+bool gui_combo_begin(const char *label, const char *preview)
 {
     bool ret;
     const theme_t *theme = theme_get();
@@ -1362,7 +1343,7 @@ bool gui_combo_begin(const char *label, const void *current)
                         ImVec2(0, (theme->sizes.item_height - font_size) / 2));
     ImGui::PushStyleColor(ImGuiCol_PopupBg, COLOR(WIDGET, INNER, 0));
     ImGui::PushItemWidth(-1);
-    ret = ImGui::BeginCombo(label, (const char*)current);
+    ret = ImGui::BeginCombo(label, preview);
 
     if (!ret) {
         ImGui::PopItemWidth();
@@ -1638,7 +1619,9 @@ bool gui_palette_entry(const uint8_t color[4], uint8_t target[4])
 
 bool gui_menu_begin(const char *label)
 {
-    return ImGui::BeginMenu(label);
+    bool ret = ImGui::BeginMenu(label);
+    if (ret) gui->capture_mouse = true;
+    return ret;
 }
 
 void gui_menu_end(void)
@@ -1646,7 +1629,7 @@ void gui_menu_end(void)
     return ImGui::EndMenu();
 }
 
-bool gui_menu_item(const char *action, const char *label, bool enabled)
+bool gui_menu_item(int action, const char *label, bool enabled)
 {
     const action_t *a = NULL;
     if (action) {
@@ -1654,7 +1637,7 @@ bool gui_menu_item(const char *action, const char *label, bool enabled)
         assert(a);
     }
     if (ImGui::MenuItem(label, a ? a->shortcut : NULL, false, enabled)) {
-        if (a) action_exec(a, "");
+        if (a) action_exec(a);
         return true;
     }
     return false;
