@@ -23,6 +23,6 @@ void gui_image_panel(void)
     image_t *image = goxel.image;
     float (*box)[4][4] = &image->box;
     gui_bbox(*box);
-    gui_action_button("img_auto_resize", "Auto resize", 0, "");
+    gui_action_button(ACTION_img_auto_resize, "Auto resize", 0);
 }
 
