@@ -66,6 +66,8 @@ struct mesh
 
 static uint64_t g_uid = 2; // Global id counter.
 
+static mesh_global_stats_t g_global_stats = {};
+
 #define N BLOCK_SIZE
 
 #define vec3_copy(a, b) do {b[0] = a[0]; b[1] = a[1]; b[2] = a[2];} while (0)
@@ -166,6 +168,8 @@ static void block_delete(block_t *block)
     block->data->ref--;
     if (block->data->ref == 0) {
         free(block->data);
+        g_global_stats.nb_blocks--;
+        g_global_stats.mem -= sizeof(*block->data);
     }
     free(block);
 }
@@ -185,6 +189,8 @@ static void block_set_data(block_t *block, block_data_t *data)
     block->data->ref--;
     if (block->data->ref == 0) {
         free(block->data);
+        g_global_stats.nb_blocks--;
+        g_global_stats.mem -= sizeof(*block->data);
     }
     block->data = data;
     data->ref++;
@@ -204,6 +210,9 @@ static void block_prepare_write(block_t *block)
     data->ref = 1;
     block->data = data;
     block->data->id = ++g_uid;
+
+    g_global_stats.nb_blocks++;
+    g_global_stats.mem += sizeof(*block->data);
 }
 
 static void block_get_at(const block_t *block, const int pos[3],
@@ -295,6 +304,7 @@ static void mesh_prepare_write(mesh_t *mesh)
         new_block = block_copy(block);
         HASH_ADD(hh, mesh->blocks, pos, sizeof(new_block->pos), new_block);
     }
+    g_global_stats.nb_meshes++;
 }
 
 static block_t *mesh_add_block(mesh_t *mesh, const int pos[3]);
@@ -306,7 +316,7 @@ static void mesh_add_neighbors_blocks(mesh_t *mesh)
         {0, -1, 0}, {0, +1, 0},
         {-1, 0, 0}, {+1, 0, 0},
     };
-    int i, p[3];
+    int i, p[3] = {};
     uint64_t key = mesh->key;
     block_t *block, *tmp, *other;
 
@@ -333,6 +343,7 @@ void mesh_remove_empty_blocks(mesh_t *mesh, bool fast)
     HASH_ITER(hh, mesh->blocks, block, tmp) {
         if (block_is_empty(block, false)) {
             HASH_DEL(mesh->blocks, block);
+            assert(mesh->blocks != block);
             block_delete(block);
         }
     }
@@ -352,6 +363,7 @@ mesh_t *mesh_new(void)
     mesh->ref = calloc(1, sizeof(*mesh->ref));
     mesh->key = 1; // Empty mesh key.
     *mesh->ref = 1;
+    g_global_stats.nb_meshes++;
     return mesh;
 }
 
@@ -405,6 +417,7 @@ void mesh_clear(mesh_t *mesh)
     mesh_prepare_write(mesh);
     HASH_ITER(hh, mesh->blocks, block, tmp) {
         HASH_DEL(mesh->blocks, block);
+        assert(mesh->blocks != block);
         block_delete(block);
     }
     mesh->key = 1; // Empty mesh key.
@@ -419,9 +432,11 @@ void mesh_delete(mesh_t *mesh)
     if (*mesh->ref == 0) {
         HASH_ITER(hh, mesh->blocks, block, tmp) {
             HASH_DEL(mesh->blocks, block);
+            assert(mesh->blocks != block);
             block_delete(block);
         }
         free(mesh->ref);
+        g_global_stats.nb_meshes--;
     }
     free(mesh);
 }
@@ -445,9 +460,11 @@ void mesh_set(mesh_t *mesh, const mesh_t *other)
     if (*mesh->ref == 0) {
         HASH_ITER(hh, mesh->blocks, block, tmp) {
             HASH_DEL(mesh->blocks, block);
+            assert(mesh->blocks != block);
             block_delete(block);
         }
         free(mesh->ref);
+        g_global_stats.nb_meshes--;
     }
     mesh->blocks = other->blocks;
     mesh->ref = other->ref;
@@ -464,11 +481,12 @@ static block_t *mesh_get_block_at(const mesh_t *mesh, const int pos[3],
                                   mesh_accessor_t *it)
 {
     block_t *block;
-    int p[3] = {pos[0] & ~(int)(N - 1),
-                pos[1] & ~(int)(N - 1),
-                pos[2] & ~(int)(N - 1)};
+    int p[3] = {};
+    p[0] = pos[0] & ~(int)(N - 1);
+    p[1] = pos[1] & ~(int)(N - 1);
+    p[2] = pos[2] & ~(int)(N - 1);
     if (!it) {
-        HASH_FIND(hh, mesh->blocks, p, 3 * sizeof(int), block);
+        HASH_FIND(hh, mesh->blocks, p, sizeof(p), block);
         return block;
     }
 
@@ -476,7 +494,7 @@ static block_t *mesh_get_block_at(const mesh_t *mesh, const int pos[3],
             vec3_equal(it->block_pos, p)) {
         return it->block;
     }
-    HASH_FIND(hh, mesh->blocks, p, 3 * sizeof(int), block);
+    HASH_FIND(hh, mesh->blocks, p, sizeof(p), block);
     it->block = block;
     it->block_id = get_block_id(block);
     vec3_copy(p, it->block_pos);
@@ -550,6 +568,18 @@ void mesh_set_at(mesh_t *mesh, mesh_iterator_t *iter,
     memcpy(BLOCK_AT(block, p[0], p[1], p[2]), v, 4);
 }
 
+void mesh_clear_block(mesh_t *mesh, mesh_iterator_t *it, const int pos[3])
+{
+    block_t *block;
+    mesh_prepare_write(mesh);
+    block = mesh_get_block_at(mesh, pos, it);
+    if (!block) return;
+    HASH_DEL(mesh->blocks, block);
+    assert(mesh->blocks != block);
+    block_delete(block);
+    if (it) it->block = NULL;
+}
+
 
 static bool mesh_iter_next_block_box(mesh_iterator_t *it)
 {
@@ -774,3 +804,9 @@ rest:
         memcpy(&data[(z * size[1] * size[0] + y * size[0] + x) * 4], v, 4);
     }
 }
+
+
+void mesh_get_global_stats(mesh_global_stats_t *stats)
+{
+    *stats = g_global_stats;
+}
